use crate::{Act, Action, Agent, Error};
use std::str::FromStr;
use xmpp_parsers::{message::Message, BareJid, Element, JidParseError};

pub struct GroupMessageAction {
    pub room: BareJid,
    pub msg: String,
    pub lang: String,
}

#[async_trait]
impl Act<()> for GroupMessageAction {
    async fn act(self, agent: &mut Agent) -> Result<(), Error> {
        agent.client.send_stanza(self.into()).await?;
        Ok(())
    }
}

impl From<GroupMessageAction> for Action {
    fn from(action: GroupMessageAction) -> Action {
        Action::GroupMessage(action)
    }
}

impl From<GroupMessageAction> for Element {
    fn from(action: GroupMessageAction) -> Element {
        let GroupMessageAction { room, msg, lang } = action;

        Message::groupchat(Some(room.into()))
            .with_body(lang, msg)
            .into()
    }
}

impl GroupMessageAction {
    /// Generates a new [`Action`] to send a group message to a parsed MUC chatroom [`BareJid`]. By default,
    /// the language used in the message is considered unknown.
    pub fn new<T: Into<BareJid>, U: AsRef<str>>(room: T, msg: U) -> GroupMessageAction {
        GroupMessageAction {
            room: room.into(),
            msg: msg.as_ref().to_string(),
            lang: "".to_string(),
        }
    }

    /// Generates a new [`Action`] to send a group message to a stringy MUC chatroom. By default,
    /// the language used in the message is considered unknown. Errors if the chatroom Jid cannot be
    /// parsed into a [`BareJid`].
    pub fn new_str<T: AsRef<str>, U: AsRef<str>>(
        room: T,
        msg: U,
    ) -> Result<GroupMessageAction, JidParseError> {
        Ok(GroupMessageAction {
            room: BareJid::from_str(room.as_ref())?,
            msg: msg.as_ref().to_string(),
            lang: "".to_string(),
        })
    }

    /// Specify the language your message is written in, if you know it.
    pub fn lang(mut self, lang: &str) -> GroupMessageAction {
        self.lang = lang.to_string();
        self
    }
}
