use crate::{Act, Action, Agent, Error, RoomNick};
use std::str::FromStr;
use xmpp_parsers::{message::Message, muc::MucUser, BareJid, Element, JidParseError};

pub struct GroupPrivateMessageAction {
    pub room: BareJid,
    pub recipient: RoomNick,
    pub msg: String,
    pub lang: String,
}

#[async_trait]
impl Act<()> for GroupPrivateMessageAction {
    async fn act(self, agent: &mut Agent) -> Result<(), Error> {
        agent.client.send_stanza(self.into()).await?;
        Ok(())
    }
}

impl From<GroupPrivateMessageAction> for Action {
    fn from(action: GroupPrivateMessageAction) -> Action {
        Action::GroupPrivateMessage(action)
    }
}

impl From<GroupPrivateMessageAction> for Element {
    fn from(action: GroupPrivateMessageAction) -> Element {
        let GroupPrivateMessageAction {
            room,
            recipient,
            msg,
            lang,
        } = action;

        let recipient = room.with_resource(recipient);

        Message::chat(Some(recipient.into()))
            .with_body(lang, msg)
            .with_payload(MucUser::new())
            .into()
    }
}

impl GroupPrivateMessageAction {
    /// Generates a new [`Action`] to send a private a parsed MUC participant (room [`BareJid`] + participant [`RoomNick`]. By default, the language used in the message is considered unknown.
    pub fn new<T: Into<BareJid>, U: AsRef<str>>(
        room: T,
        recipient: RoomNick,
        msg: U,
    ) -> GroupPrivateMessageAction {
        GroupPrivateMessageAction {
            room: room.into(),
            recipient,
            msg: msg.as_ref().to_string(),
            lang: "".to_string(),
        }
    }

    /// Generates a new [`Action`] to send a private message to a stringy MUC participant (stringy room + stringy participant).
    /// By default, the language used in the message is considered unknown. Errors if the chatroom Jid cannot be
    /// parsed into a [`BareJid`].
    pub fn new_str<T: AsRef<str>, U: AsRef<str>>(
        room: T,
        recipient: RoomNick,
        msg: U,
    ) -> Result<GroupPrivateMessageAction, JidParseError> {
        Ok(GroupPrivateMessageAction {
            room: BareJid::from_str(room.as_ref())?,
            recipient,
            msg: msg.as_ref().to_string(),
            lang: "".to_string(),
        })
    }

    /// Specify the language your message is written in, if you know it.
    pub fn lang(mut self, lang: &str) -> GroupPrivateMessageAction {
        self.lang = lang.to_string();
        self
    }
}
