use crate::{Act, Action, Agent, Error};
use std::str::FromStr;
use xmpp_parsers::{
    date::DateTime,
    muc::muc::{History, Muc},
    presence::Presence,
    BareJid, Element, JidParseError,
};

#[derive(Clone, Debug)]
pub struct JoinRoomAction {
    pub room: BareJid,
    pub nick: String,
    pub password: Option<String>,
    pub lang: Option<String>,
    pub status: Option<String>,
    pub history: History,
}

#[derive(Clone, Debug, Default)]
pub struct JoinRoomResult {
    pub nicks: Vec<String>,
    pub topic: String,
}

#[async_trait]
impl Act<JoinRoomResult> for JoinRoomAction {
    async fn act(self, agent: &mut Agent) -> Result<JoinRoomResult, Error> {
        agent.client.send_stanza(self.clone().into()).await?;

        let mut recv_presence = agent.hook_muc_presence(self.room.clone());
        let mut recv_topic = agent.hook_muc_topic(self.room.clone());
        let mut res = JoinRoomResult::default();

        info!("Waiting for room response");
        // TODO: Why deadlock?!
        //let res = tokio::spawn(async move {
        //loop {
        tokio::select! {
            hook_res = recv_presence.recv() => {
                if let Some(nick) = hook_res {
                    res.nicks.push(nick);
                }
                // break
            },
            hook_res = recv_topic.recv() => {
                if let Some((_opt_nick, subject, _lang)) = hook_res {
                    res.topic = subject.0;
                }
                //break
            }
        }
        //}

        //return res;

        //}).await.unwrap();

        Ok(res)
    }
}

impl From<JoinRoomAction> for Action {
    fn from(action: JoinRoomAction) -> Action {
        Action::JoinRoom(action)
    }
}

impl From<JoinRoomAction> for Element {
    fn from(action: JoinRoomAction) -> Element {
        let JoinRoomAction {
            room,
            nick,
            password,
            lang,
            status,
            history,
        } = action;

        let mut muc = Muc::new().with_history(history);

        if let Some(password) = password {
            muc = muc.with_password(password);
        }

        let room_jid = room.with_resource(nick);

        let mut presence = Presence::available()
            .with_to(room_jid.clone())
            .with_payload(muc);

        if let Some(status) = status {
            // lang None is treated as unknown language (empty string)
            presence.set_status(lang.unwrap_or("".to_string()), status);
        }

        presence.into()
    }
}

impl JoinRoomAction {
    /// Creates an [`Action`] to join a MUC chatroom from a parsed [`BareJid`]. By default,
    /// legacy history management is disabled. In the future, it should be replaced with MAM, but if
    /// you really need legacy history right now we still provide helpers for it.
    pub fn new<T: Into<BareJid>, U: AsRef<str>>(room: T, nick: U) -> JoinRoomAction {
        JoinRoomAction {
            room: room.into(),
            nick: nick.as_ref().to_string(),
            password: None,
            lang: None,
            status: None,
            history: History::new().with_maxchars(0),
        }
    }

    /// Creates an [`Action`] to join a MUC chatroom from a stringy bare JID. By default,
    /// legacy history management is disabled. Errors if the room JID is not parseable into a [`BareJid`].
    /// In the future, it should be replaced with MAM, but if you really need legacy history right now we still
    /// provide helpers for it.
    pub fn new_str<T: AsRef<str>, U: AsRef<str>>(
        room: T,
        nick: U,
    ) -> Result<JoinRoomAction, JidParseError> {
        Ok(JoinRoomAction {
            room: BareJid::from_str(room.as_ref())?,
            nick: nick.as_ref().to_string(),
            password: None,
            lang: None,
            status: None,
            history: History::new().with_maxchars(0),
        })
    }

    /// Sets the nick associated with the client on the chatroom to be joined.
    pub fn nick(mut self, nick: &str) -> JoinRoomAction {
        self.nick = nick.to_string();
        self
    }

    /// Sets the password required to enter the MUC chatroom.
    pub fn password(mut self, password: &str) -> JoinRoomAction {
        self.password = Some(password.to_string());
        self
    }

    /// Sets a status associated with the client, when joining the room. The status
    /// should define a language associated with it, but None will be treated as explicitly
    /// not knowing the language, i.e. `xml:lang=""` as explained [here](https://gitlab.com/xmpp-rs/xmpp-rs/-/issues/80).
    pub fn status(mut self, lang: Option<String>, status: &str) -> JoinRoomAction {
        if let Some(lang) = lang {
            self.lang = Some(lang);
        }
        self.status = Some(status.to_string());
        self
    }

    /// Request specific [`History`] settings from legacy MUC history.
    pub fn history(mut self, history: History) -> JoinRoomAction {
        self.history = history;
        self
    }

    /// Request a maximum of `maxchars` characters from legacy MUC history.
    pub fn history_maxchars(mut self, maxchars: u32) -> JoinRoomAction {
        self.history = History::new().with_maxchars(maxchars);
        self
    }

    /// Request a maximum of `maxstanzas` stanzas from legacy MUC history.
    pub fn history_maxstanzas(mut self, maxstanzas: u32) -> JoinRoomAction {
        self.history = History::new().with_maxstanzas(maxstanzas);
        self
    }

    /// Request all messages since `sec` seconds from legacy MUC history.
    pub fn history_since_sec(mut self, sec: u32) -> JoinRoomAction {
        self.history = History::new().with_seconds(sec);
        self
    }

    /// Request all messages since `since` [`DateTime`] from legacy MUC history.
    pub fn history_since(mut self, since: DateTime) -> JoinRoomAction {
        self.history = History::new().with_since(since);
        self
    }
}
