//! The [`Action`] API allows to create type-safe actions for an [`Agent`].
//!
//! You can build an Action manually by generating one of the enum variants, or use one of the associated builders:
//!
//! - [`Action::join_room`]
//! - [`Action::private_message`]
//! - [`Action::group_message`]
//! - [`Action::group_private_message`]

use crate::{Agent, Error, RoomNick};
use xmpp_parsers::{BareJid, Jid};

pub mod join_room;
pub use join_room::JoinRoomAction;
pub mod group_message;
pub use group_message::GroupMessageAction;
pub mod group_private_message;
pub use group_private_message::GroupPrivateMessageAction;
pub mod private_message;
pub use private_message::PrivateMessageAction;

pub enum Action {
    ///
    JoinRoom(JoinRoomAction),
    GroupMessage(GroupMessageAction),
    GroupPrivateMessage(GroupPrivateMessageAction),
    PrivateMessage(PrivateMessageAction),
}

impl Action {
    /// Starts building a [`JoinRoomAction`]
    pub fn join_room<T: Into<BareJid>, U: AsRef<str>>(room: T, nick: U) -> JoinRoomAction {
        JoinRoomAction::new(room, nick)
    }

    /// Starts building a [`GroupMessageAction`]
    pub fn group_message<T: Into<BareJid>, U: AsRef<str>>(room: T, msg: U) -> GroupMessageAction {
        GroupMessageAction::new(room, msg)
    }

    /// Starts building a [`GroupPrivateMessageAction`]
    pub fn group_private_message<T: Into<BareJid>, U: AsRef<str>>(
        room: T,
        nick: RoomNick,
        msg: U,
    ) -> GroupPrivateMessageAction {
        GroupPrivateMessageAction::new(room, nick, msg)
    }

    /// Starts building a [`PrivateMessageAction`]
    pub fn private_message<T: Into<Jid>, U: AsRef<str>>(room: T, msg: U) -> PrivateMessageAction {
        PrivateMessageAction::new(room, msg)
    }
}

/// The trait to implement for typed actions to be consumed by an [`Agent`].
/// It contains a single method for the action logic.
#[async_trait]
pub trait Act<T> {
    /// The method called by the [`Agent`] to perform the action.
    async fn act(self, client: &mut Agent) -> Result<T, Error>;
}
