use crate::{Act, Action, Agent, Error};
use std::str::FromStr;
use xmpp_parsers::{message::Message, Element, Jid, JidParseError};

pub struct PrivateMessageAction {
    pub jid: Jid,
    pub msg: String,
    pub lang: String,
}

#[async_trait]
impl Act<()> for PrivateMessageAction {
    async fn act(self, agent: &mut Agent) -> Result<(), Error> {
        agent.client.send_stanza(self.into()).await?;
        Ok(())
    }
}

impl From<PrivateMessageAction> for Action {
    fn from(action: PrivateMessageAction) -> Action {
        Action::PrivateMessage(action)
    }
}

impl From<PrivateMessageAction> for Element {
    fn from(action: PrivateMessageAction) -> Element {
        let PrivateMessageAction { jid, msg, lang } = action;

        Message::chat(Some(jid.into())).with_body(lang, msg).into()
    }
}

impl PrivateMessageAction {
    /// Generates a new [`Action`] to send a private message to a parsed [`Jid`]. By default,
    /// the language used in the message is considered unknown.
    pub fn new<T: Into<Jid>, U: AsRef<str>>(jid: T, msg: U) -> PrivateMessageAction {
        PrivateMessageAction {
            jid: jid.into(),
            msg: msg.as_ref().to_string(),
            lang: "".to_string(),
        }
    }

    /// Generates a new [`Action`] to send a private message to a stringy JID. By default,
    /// the language used in the message is considered unknown. Errors if the recipient Jid cannot be
    /// parsed into a [`Jid`].
    pub fn new_str<T: AsRef<str>, U: AsRef<str>>(
        jid: T,
        msg: U,
    ) -> Result<PrivateMessageAction, JidParseError> {
        Ok(PrivateMessageAction {
            jid: Jid::from_str(jid.as_ref())?,
            msg: msg.as_ref().to_string(),
            lang: "".to_string(),
        })
    }

    /// Specify the language your message is written in, if you know it.
    pub fn lang(mut self, lang: &str) -> PrivateMessageAction {
        self.lang = lang.to_string();
        self
    }
}
